package ru.dbotthepony.mc.otm.client.screen.panels.slot

import com.mojang.blaze3d.systems.RenderSystem
import com.mojang.blaze3d.vertex.PoseStack
import net.minecraft.client.renderer.GameRenderer
import net.minecraft.network.chat.Component
import net.minecraft.world.item.ItemStack
import net.minecraftforge.client.extensions.common.IClientItemExtensions
import ru.dbotthepony.mc.otm.client.render.*
import ru.dbotthepony.mc.otm.client.screen.MatteryScreen
import ru.dbotthepony.mc.otm.client.screen.panels.EditablePanel
import ru.dbotthepony.mc.otm.client.screen.panels.IItemStackPanel
import ru.dbotthepony.mc.otm.core.math.RGBAColor

abstract class AbstractSlotPanel<out S : MatteryScreen<*>> @JvmOverloads constructor(
	screen: S,
	parent: EditablePanel<*>?,
	x: Float = 0f,
	y: Float = 0f,
	width: Float = SIZE,
	height: Float = SIZE,
	open val noItemIcon: MatterySprite? = null
) : EditablePanel<S>(screen, parent, x, y, width, height), IItemStackPanel {
	protected open fun renderSlotBackground(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float) {
		SLOT_BACKGROUND.render(stack, width = width, height = height)
	}

	protected open fun renderRegular(stack: PoseStack, itemstack: ItemStack, countOverride: String? = null) {
		RenderSystem.setShader(GameRenderer::getPositionTexShader)

		if (!itemstack.isEmpty) {
			screen.renderItemStack(absoluteX, absoluteY, itemstack, countOverride)
			clearDepth(stack)
		}

		if (isHovered) {
			drawColor = SLOT_HIGHLIGHT
			drawRect(stack, 1f, 1f, width - 1, height - 1)
		}
	}

	protected open fun getItemStackTooltip(stack: ItemStack): List<Component> {
		return screen.getTooltipFromItem(stack)
	}

	override fun innerRender(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float) {
		renderSlotBackground(stack, mouseX, mouseY, partialTick)
		val itemStack = itemStack
		renderRegular(stack, itemStack)

		if (itemStack.isEmpty) {
			noItemIcon?.render(stack, width = width, height = height)
		}
	}

	override fun innerRenderTooltips(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float): Boolean {
		if (isHovered) {
			val itemstack = itemStack

			if (!itemstack.isEmpty) {
				// val font = RenderProperties.get(itemstack).getFont(itemstack)
				val font = IClientItemExtensions.of(itemstack).getFont(itemstack, IClientItemExtensions.FontContext.TOOLTIP)

				// TODO: WHERE????????????
				// GuiUtils.preItemToolTip(itemstack);
				screen.renderComponentTooltip(
					stack,
					getItemStackTooltip(itemstack),
					mouseX.toInt(),
					mouseY.toInt(),
					font ?: screen.font,
					itemstack
				)
				// GuiUtils.postItemToolTip();

				return true
			}
		}
		return false
	}

	companion object {
		val SLOT_HIGHLIGHT = RGBAColor(255, 255, 255, 100)
		val SLOT_HIGHLIGHT_DRAG = RGBAColor(200, 200, 200, 150)
		const val SIZE = 18f
		val SLOT_BACKGROUND = WidgetLocation.MISC.sprite(0f, 0f, SIZE, SIZE)
		val FILTERED_SLOT_BACKGROUND = WidgetLocation.MISC.sprite(46f, 0f, SIZE, SIZE)
	}
}
