package ru.dbotthepony.mc.otm.core.collect

import java.util.function.Consumer
import java.util.function.Predicate

class PredicateIterator<T> : Iterator<T> {
	private val parent: Iterator<T>
	private val predicate: Predicate<in T>
	private val consumer: Consumer<in T>?

	constructor(parent: Iterator<T>, predicate: Predicate<in T>) {
		this.parent = parent
		this.predicate = predicate
		this.consumer = null
	}

	constructor(parent: Iterator<T>, predicate: Predicate<in T>, consumer: Consumer<in T>) {
		this.parent = parent
		this.predicate = predicate
		this.consumer = consumer
	}

	private var foundValue: Any? = Companion

	var once: Boolean = false
		private set

	override fun hasNext(): Boolean {
		if (foundValue === Companion) {
			while (parent.hasNext()) {
				val next = parent.next()

				if (predicate.test(next)) {
					foundValue = next
					once = true
					return true
				}
			}

			return false
		}

		return true
	}

	@Suppress("unchecked_cast")
	override fun next(): T {
		if (!hasNext()) {
			throw NoSuchElementException()
		}

		val foundValue = foundValue

		if (foundValue === Companion) {
			throw ConcurrentModificationException()
		}

		this.foundValue = Companion
		consumer?.accept(foundValue as T)
		return foundValue as T
	}

	private companion object
}

fun <T> Iterator<T>.filter(condition: Predicate<in T>) = PredicateIterator(this, condition)

class MutablePredicateIterator<T> : MutableIterator<T> {
	private val parent: MutableIterator<T>
	private val predicateParent: PredicateIterator<T>

	constructor(parent: MutableIterator<T>, predicate: Predicate<in T>) {
		this.parent = parent
		this.predicateParent = PredicateIterator(parent, predicate)
	}

	constructor(parent: MutableIterator<T>, predicate: Predicate<in T>, consumer: Consumer<in T>) {
		this.parent = parent
		this.predicateParent = PredicateIterator(parent, predicate, consumer)
	}

	override fun hasNext(): Boolean {
		return predicateParent.hasNext()
	}

	override fun next(): T {
		return predicateParent.next()
	}

	override fun remove() {
		if (!predicateParent.once) {
			throw NoSuchElementException()
		}

		return parent.remove()
	}
}

fun <T> MutableIterator<T>.filter(condition: Predicate<in T>) = MutablePredicateIterator(this, condition)
