package ru.dbotthepony.mc.otm.data.loot

import com.google.gson.JsonDeserializationContext
import com.google.gson.JsonObject
import com.google.gson.JsonPrimitive
import com.google.gson.JsonSerializationContext
import com.google.gson.JsonSyntaxException
import net.minecraft.world.entity.player.Player
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.Rarity
import net.minecraft.world.level.storage.loot.LootContext
import net.minecraft.world.level.storage.loot.Serializer
import net.minecraft.world.level.storage.loot.functions.LootItemFunction
import net.minecraft.world.level.storage.loot.functions.LootItemFunctionType
import net.minecraft.world.level.storage.loot.parameters.LootContextParams
import org.apache.logging.log4j.LogManager
import ru.dbotthepony.mc.otm.core.nbt.set
import ru.dbotthepony.mc.otm.core.set
import ru.dbotthepony.mc.otm.registry.MItemFunctionTypes

enum class RandomizerFunction(val rarity: Rarity) : LootItemFunction, LootItemFunction.Builder {
	COMMON(Rarity.COMMON),
	UNCOMMON(Rarity.UNCOMMON),
	RARE(Rarity.RARE),
	EPIC(Rarity.EPIC);

	override fun apply(itemStack: ItemStack, context: LootContext): ItemStack {
		val randomizer = itemStack.item as? IRandomizableItem

		if (randomizer == null) {
			LOGGER.error("${itemStack.item} does not implement ${IRandomizableItem::class.qualifiedName}! Can not randomize $itemStack!")
			return itemStack
		}

		val random = context.random

		val player =
			context.getParamOrNull(LootContextParams.DIRECT_KILLER_ENTITY).let {
				if (it != null)
					it as? Player
				else
					context.getParamOrNull(LootContextParams.KILLER_ENTITY).let {
						if (it != null)
							it as? Player
						else
							context.getParamOrNull(LootContextParams.THIS_ENTITY) as? Player
					}
			}

		randomizer.randomize(itemStack, random, player, rarity)
		return itemStack
	}

	override fun getType(): LootItemFunctionType {
		return MItemFunctionTypes.RANDOMIZER
	}

	override fun build(): LootItemFunction {
		return this
	}

	companion object : Serializer<RandomizerFunction> {
		fun valueOf(rarity: Rarity): RandomizerFunction {
			return when(rarity) {
				Rarity.COMMON -> COMMON
				Rarity.UNCOMMON -> UNCOMMON
				Rarity.RARE -> RARE
				Rarity.EPIC -> EPIC
			}
		}

		override fun serialize(p_79325_: JsonObject, p_79326_: RandomizerFunction, p_79327_: JsonSerializationContext) {
			p_79325_["rarity"] = JsonPrimitive(p_79326_.rarity.name)
		}

		override fun deserialize(p_79323_: JsonObject, p_79324_: JsonDeserializationContext): RandomizerFunction {
			return valueOf(Rarity.valueOf(p_79323_["rarity"]?.asString ?: throw JsonSyntaxException("Invalid rarity json element")))
		}

		private val LOGGER = LogManager.getLogger()
	}
}
